(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUV language                 *)
(*                                                                        *)
(*  Copyright (C) 2004, 2005 Ralf Treinen                                 *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@lsv.ens-cachan.fr>                              *)
(*  Laboratoire Spcification et Vrification                             *)
(*  cole Normale Suprieure de Cachan, CNRS UMR 8643                     *)
(*  61, avenue du Prsident Wilson                                        *)
(*  94235 CACHAN Cedex - France                                           *)
(**************************************************************************)

(**********************************)
(*          Expressions           *)
(**********************************)


(* universal type of expressions *)
type expression =
    Ident of string
  | Int of int
  | Apply of string * expression list
  | Infix of string * expression * expression
  | Inverse of expression
  | Tuple of expression list
  | Liste of expression list
  | AssocList of (expression*expression) list

type pattern = expression list

(* conversion to string *)
val string_of_expression: expression -> string
val string_of_expression_list: expression list -> string
val string_of_pattern: pattern -> string

(* (rename_expr alpha expr) yields a copy of expr where every     *)
(* variable x defined in the association list alpha has been      *) 
(* replaced by alpha(x).                                          *)
(* rename_pattern does the same with patterns.                     *)
val rename_expr: (string * string) list -> expression -> expression
val rename_pattern: (string * string) list -> pattern -> pattern

(* (is_l_expression e) is true iff the expression e is an l-expression *)
(* that is either an identifier, or an identifier applied to some      *)
(* expressions.                                                        *)
val is_l_expression : expression -> bool 

(* collect a sorted list of all integers constants occurring in an *)
(* expression.                                                     *)
val int_constants : expression -> int list

(* (type_expr sign env sk_env r_env expr) returns the (most specific) type *)
(* of the expression expr where                                            *)
(* - sign is the signature (simple)                                        *)
(* - env is the environment                                                *)
(* - sk_env is the environment for skoelm variables                        *)
(* - r_env is an association list associating to role names its toplevel   *)
(*   declaration list. This is only use when typechecking formulas,        *)
(*   othewise this is the empty list.                                      *)
(* The result is always an arrow-free type expression without variables.   *)
exception Exc_typeexpression of string
val type_expr:
  Signatures.simple_signature
  -> Declarations.declaration list
  -> Declarations.sk_declaration list
  -> (string * (Declarations.declaration list)) list
  -> expression
  -> Types.type_expression




