(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUV language                 *)
(*                                                                        *)
(*  Copyright (C) 2004, 2005 Ralf Treinen                                 *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@lsv.ens-cachan.fr>                              *)
(*  Laboratoire Spcification et Vrification                             *)
(*  cole Normale Suprieure de Cachan, CNRS UMR 8643                     *)
(*  61, avenue du Prsident Wilson                                        *)
(*  94235 CACHAN Cedex - France                                           *)
(**************************************************************************)

(* formulas *)

open Expressions
open Declarations

type formula =
    Ftrue
  | Fatom of expression
  | Fsecret of expression
  | Fprocessat of string * string
  | Fscenarioat of string
  | Fand of formula * formula
  | For of formula * formula
  | Fimplies of formula * formula
  | Fnot of formula
  | Fforall of declaration list * formula
  | Fexists of declaration list * formula
  | Funique of declaration list * formula
  | Fbox of formula

(**** check for special syntactic forms of formulas ***********************)

(* universal closure of a quantifier-free formula *)
val is_pi1 : formula -> bool

(* safety formula : universal closure of an implication b -> g where *)
(* - b is quantifier-free                                            *)
(* - g is a  positive boolean combination of sigma1-formulas         *)
val is_safety_formula : formula -> bool

(**** type checking *******************************************************)

(* (check_types-formula sign decl_list role_toplevel_decls role_checkpoints *)
(*                      scene_checkpoints formula)                          *)
(* checks whether formula is well-typed (it returns () if it is the case    *)
(* and raise the exception Exc_formula otherwise) where                     *)
(* - sign is the signature (simple)                                         *)
(* - decl_list is the declaration list for free variables of the formula    *)
(* - role_toplevel_decls is an association list that associates to roles    *)
(*   the list of its toplevel declarations.                                 *)
(* - role_checkpoints is an association list that associates to any role    *)
(* - scene_checkpoints is the list of checkpoints of the scenario           *)
(*   its list of checkpoints.                                               *)
exception Exc_formula of string
val check_types_formula:
  Signatures.simple_signature
  -> Declarations.declaration list
  -> (string*(Declarations.declaration list)) list
  -> (string*string list) list
  -> string list
  -> formula
  -> unit
