(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUV language                 *)
(*                                                                        *)
(*  Copyright (C) 2004, 2005 Ralf Treinen                                 *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@lsv.ens-cachan.fr>                              *)
(*  Laboratoire Spcification et Vrification                             *)
(*  cole Normale Suprieure de Cachan, CNRS UMR 8643                     *)
(*  61, avenue du Prsident Wilson                                        *)
(*  94235 CACHAN Cedex - France                                           *)
(**************************************************************************)

(* the type of integers extended by omega (= infinite) *)
type omint = OmInt of int | Omega

(* the type of integer intervals *)
type interval = omint * omint

let smallerequal l r = match l with
    OmInt n ->
      (match r with
	   OmInt m -> n <= m
	 | Omega -> true)
  | Omega -> 
      (match r with
	   OmInt _ -> false
	 | Omega -> true)

let string_of_omint = function
    OmInt n -> string_of_int n
  | Omega -> "*"

let string_of_interval (left,right) =
  (string_of_omint left) ^ ".." ^ (string_of_omint right)

let string_of_interval_list = function
    [] -> ""
  | l ->
      " [ " ^
	(Utils.string_of_string_list ", "
	   (List.map string_of_interval l)) ^
	" ]"

exception Exc_intervals of string
let check_interval_list = List.iter
  (function (left,right) ->
     if not (smallerequal left right)
     then raise (Exc_intervals ((string_of_omint left) ^ " is strictly" ^
				  " greater than " ^ (string_of_omint right))))

