(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUV language                 *)
(*                                                                        *)
(*  Copyright (C) 2004, 2005 Ralf Treinen                                 *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@lsv.ens-cachan.fr>                              *)
(*  Laboratoire Spcification et Vrification                             *)
(*  cole Normale Suprieure de Cachan, CNRS UMR 8643                     *)
(*  61, avenue du Prsident Wilson                                        *)
(*  94235 CACHAN Cedex - France                                           *)
(**************************************************************************)

open Signatures
open Types
open Expressions
open Roles
open Declarations
open Axioms
open Scenarios 

type spec = signature * axioms * role list * sk_declaration list * scenario


(*************************************************************)
(*    Printing the abstract syntax                           *)
(*************************************************************)

let print_spec oc (signature, (ax_decl,ax_eqs), rolelist, vars, scenario) =
  let os = output_string oc
  in
    Signatures.print_signature oc signature;

    os "axioms\n";
    os "declare\n";
    List.iter
      (function decl ->
	 os "  ";
	 os (string_of_declaration decl);
	 os ";\n")
      ax_decl;
    os "begin\n";
    List.iter
      (function (eq) ->
	 os "  ";
	 os (string_of_expression eq);
	 os ";\n")
      ax_eqs;
    os "end\n\n";
    
    List.iter (print_role oc) rolelist;
    
    os "\nvariables\n";
    List.iter 
      (function sk_decl ->
	 os "  ";
	 os (string_of_sk_declaration sk_decl);
	 os ";\n")
      vars;	
    os "end\n\n";
    
    os "\nscenario\n";
    print_scenario oc scenario;
    os "\nend\n";
    
    close_out oc

