(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUV language                 *)
(*                                                                        *)
(*  Copyright (C) 2004, 2005 Ralf Treinen                                 *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@lsv.ens-cachan.fr>                              *)
(*  Laboratoire Spcification et Vrification                             *)
(*  cole Normale Suprieure de Cachan, CNRS UMR 8643                     *)
(*  61, avenue du Prsident Wilson                                        *)
(*  94235 CACHAN Cedex - France                                           *)
(**************************************************************************)


let singlyfy l =
  let rec singlyfy_aux last = function
      [] -> []
    | h::r ->
	if (compare last h) = 0
	then singlyfy_aux last r
	else h::(singlyfy_aux h r)
  in
    match List.sort compare l with
	[] -> []
      | h::r -> h :: (singlyfy_aux h r)

let string_of_string_list separator = function
    [] -> ""
  | h::r -> List.fold_left (fun s e -> s^separator^ e ) h r


(* (alldifferent l) checks that all elements (strings) of l are different *)
(* If a string s occurs twice in l then the exception Twice(s) is raised. *)
exception Twice of string
let rec alldifferent = function
    [] -> ()
  | h::r ->
      if (List.mem h r)
      then raise (Twice h)
      else (alldifferent r)

(* (listminus cmp l1 l2), where cmp is a compare function and l1 and l2 *)
(* are sorted according to cmp, returns (l1-l2)                         *)
let rec listminus cmp l1 l2 = match l1 with
    [] -> []
  | h1::r1 ->
      match l2 with
	  [] -> l1
	| h2::r2 -> let c = (cmp h1 h2)  in
	    if c=0 then listminus cmp r1 l2
	    else if c < 0
	    then h1::(listminus cmp r1 l2)
	    else (listminus cmp l1 r2)


let mapindex f =
  let rec mapindex_aux f n = function
      [] -> []
    | h::r -> (f n h)::(mapindex_aux f (n+1) r)
  in
    mapindex_aux f 0

exception Missing of string
let checksubset l1 l2 = 
  List.iter (function e -> if not (List.mem e l2) then raise (Missing e)) l1
